/*******************************************************************************
*  This program is a modified version of the Eg1_ADC_DMA_DAC of AN52705
*  It has been adopted for the CY8CKIT-0059 board (I. Cserny)
*
*  Project Name:     Eg1_ADC_DMA_DAC
*  Device Tested:    CY8C5888LTI-LP097
*  Software Version: PSoC Creator 4.2
*  Complier tested:  GCC 5.41
*  Related Hardware: CY8CKIT-059
********************************************************************************
* Copyright (2014)), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************


******************************************************************************* 
					PROJECT DESCRIPTION 
******************************************************************************* 
 * The project aims at explaining how to use DMA for a simple transfer data from ADC to DAC
 * The DMA transfers 8 bit data from the ADC output register to the VDAC data register. 
 * Thus the input signal given to the ADC will be reproduced at the DAC output.
 * In this example, the ADC is limited to 0 to 1.024 V input range. The VDAC is also limited to 0 to 1.024 V.
 * To observe the output, a signal should be given from the function generator to the ADC and the output
 * can be observed using an oscilloscope.
 * In this example the ADC sampling frequency(fs) is 384kHZ and the input is reconstructed best 
 * if the input frequency is less than or equal to ~84kHz because the delta sigma ADCs have a
 * low pass nature with -3dB frequency of 0.22fs. 
 *
 * Hardware connections:
 * Connect the Function generator to pin P0[0] - This is the input to ADC
 * Connect the Oscilloscope probe to pin P0[6] - This is the VDAC output
 ******************************************************************************/

#include <device.h>

/* Variable declarations for DMA . 
* These variables are defined as global variables to avoid "may be used before being set" warning 
* issued by the PSoC 5 compilers  MDK/RVDS.In this case these variables are automatically initialized to zero */

/* DMA_Chan is used to store the DMA channel handle */ 
uint8 DMA_Chan;

/* DMA_TD array is used to store all of the TDs associated with the channel
 * Since there is only one TD in this example, DMA_TD array contains only one element */
uint8 DMA_TD[1];

int main(void)
{

    /* Define for DMA Configuration */
    #define DMA_BYTES_PER_BURST 1
    #define DMA_REQUEST_PER_BURST 1
	
	/* Both source and destinations base address is peripheral base address */
    #define DMA_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_DST_BASE (CYDEV_PERIPH_BASE)

    /* Initialization of DMA */

	/* Step 1: DmaInitialize - Initialize the DMA channel
	 * Burst count = 1, (8 bit data transferred from ADC to VDAC at a time )
	 * Request per bit = 1 (Every burst transfer requires a new request)
	 * High byte of source address = Upper 16 bits of ADC data register
     * High byte of destination address =  Upper bytes of the VDAC8 data register
     * DMA_Chan holds the channel handle returned by the ‘DmaInitialize’ function. This is 
	 * used for all further references of the channel */

    DMA_Chan = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, 
                                 HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
                                 
                                 
    /* Step 2: Allocate TD */
    DMA_TD[0] = CyDmaTdAllocate();
   
    /* Step 3: CyDmaTdSetConfiguration - Configures the TD:
	 * Transfer count = 1 (Total number of bytes to transfer from the ADC to DAC)
	 * Next Td = DMA_TD[0]. The same td has to repeat itself 
	 * Configuration = No configuration required
	*/
    CyDmaTdSetConfiguration(DMA_TD[0], 1, DMA_TD[0], 0);
    
    /* Step 4: CyDmaTdSetAddress - Configure the lower 16 bit source and destination addresses 
	 * Source address = Lower 16 bits of ADC data register
	 * Destination address = Lower 16 bits of VDAC8 data register */
    CyDmaTdSetAddress(DMA_TD[0], LO16((uint32)ADC_DelSig_DEC_SAMP_PTR), LO16((uint32)VDAC8_Data_PTR));
    
    /* Step 5: Map the TD to the DMA Channel */
    CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
    
    /* Step 6: Enable the channel
	 * The Channel is enabled with preserveTds set to 1. This will reload the original TD settings 
	 * after the transfer is complete so that TD can be repeated */
    CyDmaChEnable(DMA_Chan, 1);

	/* Start ADC and DAC */
	ADC_DelSig_Start();
	VDAC8_Start();
	
	/* Start ADC Conversion */
	ADC_DelSig_StartConvert();
	
	for(;;)
    {
        /* Your code here. */
        /* The ADC data is moved to the DAC by the DMA in the background.
         * No CPU intervention is required in this project */

    }
}

/* [] END OF FILE */
